/*
 * Copyright (c) 2001 Mark Fullmer and The Ohio State University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      $Id: flow-receive.c,v 1.5 2001/02/10 04:40:12 maf Exp $
 */

#include <sys/time.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/signal.h>
#include <sys/errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <time.h>
#include <fcntl.h>
#include "ftlib.h"
#include "fmt.h"

int debug;
int done;

void usage();
void sig_quit();

int main(argc, argv)
int argc;
char **argv;
{

  extern char *optarg;
  extern int optind, errno;
  int fport, fsockfd, len, i, n, line_buf;
  fd_set rfd;
  struct sockaddr_in fsocaddr;
  extern int done;
  char rcv_buf[FT_RCV_BUFSIZE];
  char fmt_buf[32];
  struct ftio ftio;
  struct ftdecode ftd;
  struct ftver ftv;
  struct ftset ftset;
  int offset;
  u_int64 nflows;

  nflows = 0;
  bzero(&ftset, sizeof ftset);
  fport = FT_PORT;
  line_buf = 1;
  ftset.z_level = 0;
  done = 0;
  bzero(&ftv, sizeof ftv);
#if BYTE_ORDER == BIG_ENDIAN
  ftset.byte_order = FT_HEADER_BIG_ENDIAN;
#endif /* BYTE_ORDER == BIG_ENDIAN */
#if BYTE_ORDER == LITTLE_ENDIAN
  ftset.byte_order = FT_HEADER_LITTLE_ENDIAN;
#endif /* BYTE_ORDER == LITTLE_ENDIAN */

  while ((i = getopt(argc, argv, "b:C:d:h?lp:z:")) != -1)

    switch (i) {

    case 'b': /* output byte order */
      if (!strcasecmp(optarg, "little"))
        ftset.byte_order = FT_HEADER_LITTLE_ENDIAN;
      else if (!strcasecmp(optarg, "big"))
        ftset.byte_order = FT_HEADER_BIG_ENDIAN;
      else {
        fprintf(stderr, "expecting \"big\" or \"little\" at -b\n");
        exit (1);
      }
      break;

    case 'C': /* comment field */
      if (strlen(optarg) > (FT_HEADER_CMNT_LEN-1)) {
        fprintf(stderr, "comments limited to %d bytes.\n", 
          (FT_HEADER_CMNT_LEN-1));
        exit (1);
      }
      strcpy(ftset.comments, optarg);
      break;

    case 'd': /* debug */
      debug = atoi(optarg);
      break;

    case 'h': /* help */
    case '?':
      usage();
      exit (0);
      break;

    case 'l': /* turn off buffered output */
      line_buf = 0;
      break;

    case 'p': /* udp port */
      fport = atoi(optarg);
      break;

    case 'z': /* compress level */
      ftset.z_level = atoi(optarg);
      if ((ftset.z_level < 0) || (ftset.z_level > 9)) {
        fprintf(stderr, "Compression level must be between 0 and 9\n");
        exit (1);
      }
      break;

    default:
      usage();
      exit (1);
      break;

    } /* switch */

  /* handle SIGQUIT */
  signal(SIGQUIT, sig_quit);
  signal(SIGHUP, sig_quit);
  signal(SIGINT, sig_quit);
  signal(SIGTERM, sig_quit);

  /* get hostname */
  if (gethostname((char*)&ftset.hostname, (int)FT_HEADER_HN_LEN-1) == -1) {
    fprintf(stderr, "gethostname(): %s\n", strerror(errno));
    exit (1);
  }

  ftset.hostname[FT_HEADER_HN_LEN-1] = 0;

  bzero(&fsocaddr, sizeof(fsocaddr));
  fsocaddr.sin_family = AF_INET;
  fsocaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  fsocaddr.sin_port = htons(fport);

  if ((fsockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    fprintf(stderr, "socket(): %s\n", strerror(errno));
    exit (1);
  }

  if (bind(fsockfd, (struct sockaddr*)&fsocaddr, sizeof(fsocaddr)) < 0) {
    fprintf(stderr, "bind(): %s\n", strerror(errno));
    exit (1);
  }

  len = sizeof(fsocaddr);

  /* output to stdout */
  if (ftio_init(&ftio, 1, FT_IO_FLAG_NO_SWAP | FT_IO_FLAG_WRITE |
    ((ftset.z_level) ? FT_IO_FLAG_ZINIT : 0) ) < 0) {
    fprintf(stderr, "ftio_init(): failed\n");
    exit (1);
  }

  ftio_set_comment(&ftio, ftset.comments);
  ftio_set_hostname(&ftio, ftset.hostname);
  ftio_set_byte_order(&ftio, ftset.byte_order);
  ftio_set_z_level(&ftio, ftset.z_level);
  ftio_set_debug(&ftio, debug);


  while (1) {

    FD_ZERO (&rfd);
    FD_SET (fsockfd, &rfd);

    if (select (fsockfd+1, &rfd, (fd_set *)0, (fd_set *)0, 
      (struct timeval*)0L) < 0)  {
      if (errno != EINTR) {
        fprintf(stderr, "select(): %s\n", strerror(errno));
        exit (1);
      }
    }

    if (done) {
      fprintf(stderr, "Cleaning up.\n");
      break;
    }

    if (FD_ISSET(fsockfd, &rfd)) {

      if ((n = recvfrom(fsockfd, (char*)&rcv_buf, sizeof rcv_buf,
        0, (struct sockaddr*)&fsocaddr, &len)) == -1) {
        fprintf(stderr, "recvfrom(): %s\n", strerror(errno));
        exit (1);
      }

      /* convert and write */
      n = fts2rec_pdu_decode(&rcv_buf, &ftd, ftset.byte_order);

      /* first time? */
      if (!ftv.d_version) {

        /* set the version information in the io stream */
        ftio_set_ver(&ftio, &ftd.ver);

        /* copy to compare next time */
        bcopy(&ftd.ver, &ftv, sizeof ftv);

        /* header first */
        if (ftio_write_header(&ftio) < 0) {
          fprintf(stderr, "ftio_write_header(): failed\n");
          exit (1);
        }

      } else {

        /* ensure previous version == current decoded pdu version */
        if ((ftv.d_version != ftd.ver.d_version) ||
            (ftv.agg_method != ftd.ver.agg_method)) {
          fprintf(stderr, "data version or sub version changed!\n");
          exit (1);
        }

      }

      for (i = 0, offset = 0; i < n; ++i, offset += ftd.rec_size) {

        ++nflows;

        if (ftio_write(&ftio, ftd.buf+offset) < 0) {
          fprintf(stderr, "ftio_write(): failed\n");
          exit (1);
        }

      }

    } /* if FD_ISSET */
  } /* while 1 */

  fmt_uint64(fmt_buf, nflows, FMT_JUST_LEFT);
  fprintf(stderr, "%s flows processed\n", fmt_buf);

  if (ftio_close(&ftio) < 0) {
    fprintf(stderr, "ftio_close(): failed\n");
    exit (1);
  }

  return 0;

} /* main */

void usage() {

  fprintf(stderr, "flow-receive:\n\n");
  fprintf(stderr, " -b s set output byte order (little or big)\n");
  fprintf(stderr, " -C s set comment field in output\n");
  fprintf(stderr, " -d # set debug level.\n");
  fprintf(stderr, " -p # udp port to receive on.\n");
  fprintf(stderr, "\n\n");

} /* usage */

void sig_quit(sig)
int sig;
{
  extern int done;
  done = 1;
} /* sig_quit */

